EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'ProGetPackages_CreateOrUpdatePackage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new ProGet package or updates an existing one.'
GO

IF OBJECT_ID('[ProGetPackages_CreateOrUpdatePackage]') IS NOT NULL DROP PROCEDURE [ProGetPackages_CreateOrUpdatePackage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ProGetPackages_CreateOrUpdatePackage]
(
	@Feed_Id INT,
	@Group_Name NVARCHAR(200),
	@Package_Name NVARCHAR(200),
	@Version_Text VARCHAR(255),
	@PackageMetadata_Bytes VARBINARY(MAX),
	@Published_Date DATETIME,
	@PackageHash_Bytes BINARY(20),
	@Cached_Indicator YNINDICATOR,
	@Package_Size BIGINT,
	@Title_Text NVARCHAR(100),
	@Description_Text NVARCHAR(MAX),
	@IconUrl_Text NVARCHAR(100),
	@LatestVersion_Text NVARCHAR(50),
	@Virtual_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @ProGetPackage_Id INT
		SET @ProGetPackage_Id = (SELECT [ProGetPackage_Id] FROM [ProGetPackages] WITH (UPDLOCK) WHERE [Feed_Id] = @Feed_Id AND [Group_Name] = @Group_Name AND [Package_Name] = @Package_Name)

		IF @ProGetPackage_Id IS NULL
		BEGIN

			INSERT INTO [ProGetPackages]
			(
				[Feed_Id],
				[Group_Name],
				[Package_Name],
				[Title_Text],
				[Description_Text],
				[IconUrl_Text],
				[LatestVersion_Text],
				[Download_Count]
			)
			VALUES
			(
				@Feed_Id,
				@Group_Name,
				@Package_Name,
				@Title_Text,
				@Description_Text,
				@IconUrl_Text,
				@LatestVersion_Text,
				0
			)

			SET @ProGetPackage_Id = SCOPE_IDENTITY()

		END
		ELSE IF @Version_Text = @LatestVersion_Text
		BEGIN

			UPDATE [ProGetPackages]
			   SET [Title_Text] = @Title_Text,
			       [Description_Text] = @Description_Text,
				   [IconUrl_Text] = @IconUrl_Text,
				   [LatestVersion_Text] = @LatestVersion_Text
			 WHERE [ProGetPackage_Id] = @ProGetPackage_Id

		END
		ELSE
		BEGIN

			UPDATE [ProGetPackages]
			   SET [LatestVersion_Text] = @LatestVersion_Text
			 WHERE [ProGetPackage_Id] = @ProGetPackage_Id

		END

		IF EXISTS(SELECT * FROM [ProGetPackageVersions] WITH (UPDLOCK) WHERE [ProGetPackage_Id] = @ProGetPackage_Id AND [Version_Text] = @Version_Text)
		BEGIN

			UPDATE [ProGetPackageVersions]
			   SET [PackageMetadata_Bytes] = COALESCE(@PackageMetadata_Bytes, [PackageMetadata_Bytes]),
			       [Published_Date] = COALESCE(@Published_Date, [Published_Date]),
				   [PackageHash_Bytes] = COALESCE(@PackageHash_Bytes, [PackageHash_Bytes]),
				   [Package_Size] = COALESCE(@Package_Size, [Package_Size]),
				   [Cached_Indicator] = COALESCE(@Cached_Indicator, [Cached_Indicator]),
				   [Virtual_Indicator] = COALESCE(@Virtual_Indicator, [Virtual_Indicator])
			 WHERE [ProGetPackage_Id] = @ProGetPackage_Id
			   AND [Version_Text] = @Version_Text

			IF COALESCE(dbo.Session_GetUserName(), 'UNKNOWN') NOT IN ('SYSTEM', 'UNKNOWN')
	   			EXEC [Events_RaiseEvent] 'PKGMDF', @Feed_Id, 'Package_Id', @Package_Name, 'Package_Version', @Version_Text

		END
		ELSE
		BEGIN

			INSERT INTO [ProGetPackageVersions]
			(
				[ProGetPackage_Id],
				[Version_Text],
				[PackageHash_Bytes],
				[PackageMetadata_Bytes],
				[Published_Date],
				[Package_Size],
				[Download_Count],
				[Cached_Indicator],
				[Virtual_Indicator]
			)
			VALUES
			(
				@ProGetPackage_Id,
				@Version_Text,
				@PackageHash_Bytes,
				@PackageMetadata_Bytes,
				COALESCE(@Published_Date, GETUTCDATE()),
				@Package_Size,
				0,
				COALESCE(@Cached_Indicator, 'N'),
				COALESCE(@Virtual_Indicator, 'N')
			)

			EXEC [Events_RaiseEvent] 'PKGADD', @Feed_Id, 'Package_Id', @Package_Name, 'Package_Version', @Version_Text

		END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ProGetPackages_CreateOrUpdatePackage] TO [ProGetUser_Role]
GO